package jp.co.sds_corp.morphogenesis;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Arc2D;
import java.io.Serializable;

public class PointMap implements Serializable
{
	int x;
	int y;
	int[][] value;
	int[][] mark;

	public PointMap(int x, int y) {
		this.x = x;
		this.y = y;
		value = new int[x][y];
		mark = new int[x][y];
		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				mark[i][j] = -1;
			}
		}
	}

	public PointMap(PointMap org) {
		x = org.x;
		y = org.y;
		value = new int[org.x][org.y];
		mark = new int[org.x][org.y];
		for (int i = 0; i < org.x; i++) {
			for (int j = 0; j < org.y; j++) {
				value[i][j] = org.value[i][j];
				mark[i][j] = org.mark[i][j];
			}
		}
	}

	public int getX() {
		return x;
	}

	public void setX(int x) {
		this.x = x;
	}

	public int getY() {
		return y;
	}

	public void setY(int y) {
		this.y = y;
	}

	public int get(int x, int y) {
		return value[x][y];
	}

	public void set(int x, int y, int value) {
		this.value[x][y] = value;
	}

	public void mark(int x, int y, int value) {
		mark[x][y] = value;
	}

	public void mark(int x, int y) {
		mark(x, y, 1 - value[x][y]);
	}

	public int getMark(int x, int y) {
		return mark[x][y];
	}

	public void doMark() {
		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				if (mark[i][j] >= 0) {
					value[i][j] = mark[i][j];
				}
			}
		}
	}

	public void reset() {
		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				mark[x][y] = -1;
			}
		}
	}

	public void paint(Graphics gr) {
		Graphics2D gr2D = (Graphics2D )gr;
		Rectangle rect = new Rectangle(1000, 1000);
		double ux = rect.getWidth() / x;
		double uy = rect.getHeight() / y;

		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				double vx = ux * i;
				if ((j % 2) != 0) {
					vx = vx + (ux * 0.5);
				}
				double vy = uy * j;
				Arc2D arc = new Arc2D.Double(Arc2D.PIE);
				if (value[i][j] == 0) {
					gr.setColor(Color.white);
				} else {
					gr.setColor(Color.black);
				}
				arc.setArc(vx, vy, ux, uy, 0, 360, Arc2D.PIE);
				gr2D.fill(arc);
			}
		}
	}

}
