package jp.co.sds_corp.morphogenesis;

import java.awt.BorderLayout;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class TuringPatternFrame extends JFrame
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	JPanel panel;
	JButton btnInit;
	JButton btnStart;
	JComboBox<String> cmbHist;
	TuringPattern turing;
	PointMap pointMap;
	List<PointMap> listHistory = new ArrayList<PointMap>();

	public void init() throws Exception {
		createComponent();

		turing = new TuringPattern(100, 100);
		turing.init();
		clearHistory();
	}

	void createComponent() {
		setSize(1030, 1080);
		setLayout(new BorderLayout());
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent ev) {
				System.exit(0);
			}
		});

		panel = new JPanel() {
			@Override
			public void paint(Graphics gr) {
				pointMap.paint(gr);
			}
		};
		add(panel, BorderLayout.CENTER);

		btnInit = new JButton("Init");
		btnInit.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ev) {
				try {
					turing.init();
				} catch (Exception ex) {
					ex.printStackTrace();
				}
				repaint();
				clearHistory();
			}
		});
		btnStart = new JButton("Start");
		btnStart.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ev) {
				turing.start();
				addHistory();
				cmbHist.setSelectedIndex(listHistory.size() - 1);
			}
		});
		cmbHist = new JComboBox<String>();
//		cmbHist.setMaximumRowCount(1);
		cmbHist.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ev) {
				int index = cmbHist.getSelectedIndex();
				if (index < 0 || index >= listHistory.size()) {
					return;
				}
				pointMap = listHistory.get(index);
				repaint();
			}
		});

		Box boxBtn = new Box(BoxLayout.X_AXIS);
		boxBtn.add(btnInit);
		boxBtn.add(btnStart);
		boxBtn.add(cmbHist);
		add(boxBtn, BorderLayout.NORTH);
	}

	void clearHistory() {
		listHistory.clear();
		listHistory.add(new PointMap(turing.getPointMap()));
		pointMap = turing.getPointMap();
		cmbHist.removeAllItems();
		cmbHist.addItem("[init]");
	}

	void addHistory() {
		listHistory.add(new PointMap(turing.getPointMap()));
		cmbHist.addItem("History[" + (listHistory.size()-1) + "]");
	}

	public void start() {
		turing.start();
		repaint();
	}

	public static void main(String[] args) {
		try {
			TuringPatternFrame frame = new TuringPatternFrame();
			frame.init();
			frame.setVisible(true);
		} catch (Exception ex) {
			logger.error(TuringPatternFrame.class, "error", ex);
			System.exit(9);
		}
	}
}
